import 'dart:async';

import 'package:flutter/material.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/osam.dart';

part 'navigation_presenter.dart';
part 'navigation_state.g.dart';
part 'osam_navigator.dart';

void registerNavigationAdapters() {
  Hive.registerAdapter(NavigationStateAdapter());
  Hive.registerAdapter(TabNavigationStateAdapter());
}

@HiveType(typeId: 0)
// ignore: must_be_immutable
class NavigationState extends BaseState<NavigationState> {
  @HiveField(0)
  var _routes = <String>[];
  final String _initialRoute;

  NavigationState(this._initialRoute) {
    _routes.add(_initialRoute);
  }

  StateStream<List<String>> get routes => propertyStream((state) => state._routes);

  void _push(String route) => _routes.add(route);

  void _pushReplacement(String route) =>
      update((_) => _routes.replaceRange(_routes.length - 1, _routes.length, [route]));

  void _pushAndRemoveUntil(String route) => update((_) => _routes
    ..clear()
    ..add(route));

  void _pop() => update((_) => _routes.removeLast());

  void toInitialRoute() {
    update((_) => _routes
      ..clear()
      ..add(_initialRoute));
  }

  @override
  List<Object> get props => [_routes];
}

@HiveType(typeId: 1)
// ignore: must_be_immutable
class TabNavigationState extends BaseState<TabNavigationState> {
  @HiveField(0)
  String _tab;

  TabNavigationState(this._tab);

  StateStream<String> get tab => propertyStream((state) => state._tab);

  void _changeTab(String tab) => _tab = tab;

  @override
  List<String> get props => [_tab];
}
